#!/usr/bin/perl
##########################################################################
#                                                                        #
# Perl agi Script for Authenticate caller via RADIUS                     #
#                                                                        #
# Modified on 110325 by yannick.vaucher@switzernet.com                   #
#                                                                        #
# Switzernet(c)2011                                                      #
#                                                                        #
##########################################################################
# my modules
use Crypt::CBC;
use Asterisk::AGI;
use Authen::Radius;
use Sys::Syslog;
use Switch;
use List::Util;
use Config::IniFiles;
use Date::Format;
use Time::HiRes qw/gettimeofday/;
use DBI;

use constant ROUTING => "SIP"; 
use constant DEBUG => 0;

use constant {H323_CONTINUE     => 0,
              H323_INVALID_ACC  => 1,
              H323_ACC_EXPIRED  => 5,
              H323_CREDIT_LIMIT => 6,
              H323_ACC_BLOCKED  => 7,
              H323_TECH_PROB    => 8,
              H323_DEST_BLOCKED => 9,
              H323_FREE_CALL    => 13
             };

use constant {CONTACT_PREFIX => 1,
              CONTACT_IP     => 0};


use constant CONFIG_DIR => "/etc/astrad/config";

##########################################################################
# Globals
my $credit_time = -1;
my $hangup = 1;
my $return_code;
my @routes;
my @Expire;
my @Join_Expire;
my $numroutes=0;
my $skip_route=0;
my $iexpires=0;
my %expires;
my $auth_cli;

my $date_format = "%Y-%m-%d %T";


my ($setuptime1,$setupms1) = gettimeofday();
my ($setuptime2,$setupms2);
my ($setuptime3,$setupms3);


##########################################################################
# Database

my $dbh;
my $LOCAL_HOST;
my $DB_NAME1;
my $DB_NAME2;
my $DB_HOST;
my $DB_USER;
my $DB_PASS;

##########################################################################
#                                FUNCTION                                #
##########################################################################

# Load config
sub load_config {
    my $conf=Config::IniFiles->new(-file => CONFIG_DIR ."/switzer.conf");
    return 0 if !defined $conf ;
    $LOCAL_HOST  = $conf->val('GLOBAL','NAS_IP');
    $DB_NAME1 = $conf->val('DB2','DB_NAME1');
    $DB_NAME2 = $conf->val('DB2','DB_NAME2');
    $DB_HOST  = $conf->val('DB2','DB_HOST');
    $DB_USER  = $conf->val('DB2','DB_USER');
    $DB_PASS  = $conf->val('DB2','DB_PASS');
    return 1;
}

sub unescape {
    shift() if ref($_[0]);
    my $todecode = shift;
    return undef unless defined($todecode);
    #$todecode =~ tr/+/ /;       # pluses become spaces
    $todecode =~ s/%([0-9a-fA-F]{2})/pack("c",hex($1))/ge;
    return $todecode;
}

###################################################################################
#
# Authenticate the client using db2 instead of Radius 
# Also return client data.
#
# params : - username of client
#
# return : - h323-return-code :
#            o H323_TECH_PROB if database is unavailable
#            o H323_INVALID_ACC if acc doesn't exist in db
#            o H323_ACC_BLOCKED if acc or customer is blocked
#            o H323_ACC_EXPIRED if acc is expired
#            o H323_CONTINUE else
#
#          - credit state : 
#            o 1 if client has credits
#            o 0 else
#
sub alternative_acc_auth {
        
    my $cli = @_[0];
	
    if (not defined $dbh){
	# Not available
        syslog('err', "Ast-Aut: Could not connect to database $DB_NAME1 on $DB_HOST");
        return (H323_TECH_PROB);
    }


    ###################################################################################
    #
    # Check the caller to know if he is allowed to call or if he is blocked
    #

    my $request="SELECT c.balance < c.credit_limit AS 'has_credits', "
                ."    (c.blocked = 'Y' OR a.blocked = 'Y') AS 'is_blocked', "
                ."    (a.expiration_date < CURDATE()) or false AS 'is_expired' "
                ." FROM Accounts a "
                ." INNER JOIN Customers c "
                ."    ON (a.i_customer = c.i_customer) "
                ." WHERE a.id='$cli';";
        
    my $sth = $dbh->prepare($request);
    $sth->execute();
        
    return (H323_INVALID_ACC) unless (my $result1 = $sth->fetchrow_hashref());

    my $has_credits = $result1->{has_credits};


    my @debug_data;

    # Additional log info
    if (DEBUG) {
        # if in debug mode request more details
        my $dbg_req="SELECT c.balance AS 'balance', c.credit_limit AS 'credit_limit'," 
                    ."    c.blocked AS 'cust_blocked', a.blocked AS 'acc_blocked'," 
                    ."    a.expiration_date AS 'expires', CURDATE() AS 'cur_date'" 
                    ."  FROM Accounts a" 
                    ."    INNER JOIN Customers c" 
                    ."    ON (a.i_customer = c.i_customer)" 
                    ." WHERE a.id='$cli';";
       
        $sth = $dbh->prepare($dbg_req);
          
        $sth->execute();

        return (H323_INVALID_ACC) unless(@debug_data = $sth->fetchrow());

    }
	

    # Account or customer is blocked
    return (H323_ACC_BLOCKED,undef,@debug_data) if ($result1->{is_blocked});

    # Account is expired
    return (H323_ACC_EXPIRED,undef,@debug_data) if ($result1->{is_expired});

    return (H323_CONTINUE,$has_credits,@debug_data);
}


###################################################################################
#
# Authenticate a node using db2 instead of Radius
#
# params : - ip of node
#
# return : - h323-return-code :
#            o H323_TECH_PROB if database is unavailable
#            o H323_INVALID_ACC if acc doesn't exist in db
#            o H323_ACC_BLOCKED if acc or customer is blocked
#            o H323_ACC_EXPIRED if acc is expired
#            o H323_CONTINUE else
#
#          - credit state :
#            o 1 if client has credits
#            o 0 else
#
sub alternative_ip_auth {

    my $node = @_[0];

    if (not defined $dbh){
        # Not available
        syslog('err', "Ast-Aut: Could not connect to database $DB_NAME1 on $DB_HOST");
        return (H323_TECH_PROB);
    }

    ###################################################################################
    #
    # Check if the node is allowed to call
    #
    my $request="SELECT ip,name
                   FROM Nodes
                 WHERE ip = '$node'
                   AND i_env=1
                   AND i_node_type=11
                   AND name not like '%[Deleted]%'";
    my $sth = $dbh->prepare($request);
    $sth->execute();

    return H323_INVALID_ACC unless (my $result1 = $sth->fetchrow_hashref());

    return H323_CONTINUE;
}
###################################################################################
#
# Define the route to destinations
#
# params : - destination
#
#
# return : - h323 return code
#          - formated destination 
#          - a list of ip to contact for routing
#
sub define_routing_params {


    # return values
    my ($return_code, $dest, @contacts);

    my $dest = @_[0];

    ############################################################################
    # Do the regexp rules depending on number called
    # It is important to match a format like +cc num
    # If it begins by the national prefix (0) replace it by 41
    # TODO use customer's defined rules

    # remove 00 and +
    $dest =~ s/^(00|\+)//;
    $dest =~ s/^0/41/;
 

    ############################################################################
    #
    # Check the dest to know if it is a free call or not
    #

    my $request="SELECT id FROM Accounts WHERE id='$dest';";
    $sth = $dbh->prepare($request);
    $sth->execute();
    my $acc_data = $sth->fetchrow_hashref();

    my $is_outgoing = not defined $acc_data->{id};

    if ($is_outgoing) {

        # set outgoing vendors by default 
        @contacts = (["212.249.15.9","+"], # Verizon 1 +dest
                     ["212.249.15.4","+"], # Verizon 2 +dest
                     ["212.249.15.5","00"], # Verizon SS7 00dest
                     ["217.168.45.4","00"]  # Colt SS7 00dest
                    );
        $return_code = H323_CONTINUE;
    } else {
        # Find and define the right SIP server
        my $request="SELECT domain FROM `$DB_NAME2`.location 
                       WHERE username='$dest' ORDER BY last_modified";
        my $sth = $dbh->prepare($request);
        $sth->execute();
        my $location = $sth->fetchrow_hashref();
        # dest format unchanged
        @contacts = ([$location->{domain},""]);
        $return_code = H323_FREE_CALL;
    }
    return ($return_code, $dest, \@contacts);

}

sub set_routing_data {
       
        my $cli = @_[0]; 
        my $dest =  @_[1];
        my $contacts_ref = @_[2];

        my @contacts = @$contacts_ref;

        my $dial_info = '';

        $num_contacts = @contacts;

        $AGI->set_variable('NumRoutes',$num_contacts);
        $AGI->set_variable('DNID',$dest);

        my $join_expire = 1;
        my $expire = 300;
        my $XSUM = 0;

        foreach my $c (@contacts){

            $dial_info .= '&' if $dial_info ne '';
            $dial_info .= ROUTING."/".$c->[CONTACT_PREFIX].$dest;
            $dial_info .= ":$secret" if defined $route_info{'auth'} && defined $secret;
            $dial_info .= ":$authuser" if defined $route_info{'auth'} && defined $authuser;
            $dial_info .= '@'.$c->[CONTACT_IP];

            my $route = ROUTING."/".$c->[CONTACT_PREFIX].$dest."@".$c->[CONTACT_IP];

            $AGI->set_variable("XROUTE_".$join_expire."_".$XSUM, $route);
            $AGI->set_variable("XEXP_".$join_expire."_".$XSUM, $expire);
            $AGI->set_variable("XCLI_".$join_expire."_".$XSUM, $cli);
            syslog('info', "Ast-Aut-Alt: XROUTE_".$join_expire."_".$XSUM."=".$route
                         ." | XEXP_".$join_expire."_".$XSUM."=".$expire
                         ." | XCLI_".$join_expire."_".$XSUM."=".$cli) ;
            $XSUM++;
        }

        $AGI->set_variable("XSUM_$join_expire", $XSUM);
        $AGI->set_variable("II", 1);
        $AGI->set_variable("XTOT", $join_expire);


        $AGI->set_variable('ActiveFolloMe', 1);
        syslog('info', "Ast-Aut-Alt: XSUM = $XSUM");
        syslog('info', "Ast-Aut-Alt: II = 1");
        syslog('info', "Ast-Aut-Alt: XTOT = 1");
        $AGI->set_variable('RADIUS_Status', 'OK');
        $AGI->set_variable('Dial_Info', $dial_info) if $dial_info ne '';
        $AGI->verbose("  -- AGI full dial info: $dial_info",4);

}


##########################################################################
#                                 main()                                 #
##########################################################################
$AGI = new Asterisk::AGI;
my %input = $AGI->ReadParse();
openlog('ast-auth', 'cons,pid', 'daemon');
syslog('notice', "Ast-Aut: Script Asterisk Authentication Started");


my $username;

my %params;
my(@pairs) = split(/[&;]/,$ARGV[0]);
my($param,$value);
foreach (@pairs) {
    ($param,$value) = split('=',$_,2);
    $param = unescape($param);
    $value = unescape($value);
    $params{$param}=$value;
}
##########################################################################
# Sending auth. radius packet to portaone server
$hangup = 0 if defined $params{'IfFailed'} && $params{'IfFailed'} =~ /DoNotHangup/i;
my ($a,$b,$c,$d) = ($1,$2,$3,$4) if defined $params{'H323_ID'} && $params{'H323_ID'} =~ /([0-9]+)-([0-9]+)-([0-9]+)-([0-9]+)/;
my $h323_id = sprintf("%08X %08X %08X %08X",$a,$b,$c,$d);
$AGI->verbose("H323 hex: $h323_id");

my $max_duration = $AGI->get_variable('max_duration');
my $r = new Authen::Radius(Host => $AGI->get_variable('RADIUS_Server'), Secret => $AGI->get_variable('RADIUS_Secret'));

if( !defined $r ) {
        $AGI->verbose('RADIUS server "'.$AGI->get_variable('RADIUS_Server').'" ERROR', 0);
        $AGI->hangup() if $hangup;
        syslog('err', "Ast-Aut: Error in Portaone Radius connection");
        closelog;
        exit;
}

Authen::Radius->load_dictionary;

$input{'callerid'} = $1 if defined $input{'callerid'}  && $input{'callerid'} =~ /<(\d*)>/;
$input{'dnid'} = $1 if defined $input{'dnid'}  && $input{'dnid'} =~ /<(\d*)>/;


$r->add_attributes (
    { Name => 'NAS-IP-Address', Value => $AGI->get_variable('NAS_IP_Address') },
    { Name => 'NAS-Port-Name', Value => $input{'channel'} },
    { Name => 'Calling-Station-Id', Value => $input{'callerid'} },
    { Name => 'Called-Station-Id', Value => $input{'dnid'} },
    { Name => 'Cisco-AVPair', Value => "call-id=$input{'uniqueid'}" },
    #{ Name => 'Cisco-AVPair', Value => "call-id=$callid" },
    { Name => 'Cisco-AVPair', Value => "h323-conf-id=$h323_id" }
);

if( defined $params{'AuthorizeBy'} && $params{'AuthorizeBy'} =~ /sip/i ) {
    my $sip_auth_header = $AGI->get_variable('SIP_Authorization');
    if( !defined $sip_auth_header ) {
        $AGI->verbose("ERROR Authorization=SIP requested but no SIP Authorization header provided.", 0);
        $AGI->set_variable('RADIUS_Status', 'ConfigurationError');
        $AGI->hangup() if $hangup;
        syslog('err', "Ast-Aut: ERROR Authorization=SIP requested but no SIP Authorization header provided.");
        closelog;
        exit;
    }
    $sip_auth_header =~ s/\s*Digest\s*//;
    my %sip_auth;
    my(@pairs) = split(/,\s?/,$sip_auth_header);
    my($param,$value);
    foreach (@pairs) {
        ($param,$value) = split('=',$_,2);
        $param = unescape($param);
        $value = unescape($value);
        $value =~ s/^"//;
        $value =~ s/"$//;
        $sip_auth{$param}=$value;
    }

    $username = $sip_auth{'username'};
    
    $r->add_attributes ( { Name => 'User-Name', Value => $username } );
    $r->add_attributes ( { Name => 'Digest-Attributes', Value => 'User-Name = "'.$username.'"' } );
    $r->add_attributes ( { Name => 'Digest-Attributes', Value => 'Realm = "'.$sip_auth{'realm'}.'"' } );
    $r->add_attributes ( { Name => 'Digest-Attributes', Value => 'Nonce = "'.$sip_auth{'nonce'}.'"' } );
    $r->add_attributes ( { Name => 'Digest-Attributes', Value => 'URI = "'.$sip_auth{'uri'}.'"' } );
    $r->add_attributes ( { Name => 'Digest-Attributes', Value => 'Method = "INVITE"' } ); 
    $r->add_attributes ( { Name => 'Digest-Attributes', Value => 'Algorithm = "'.$sip_auth{'algorithm'}.'"' } );
    $r->add_attributes ( { Name => 'Digest-Response', Value => $sip_auth{'response'} } );
    $AGI->set_variable('SIP_Username', $username);
} else {
    #TODO looks wrong, check
    $username = $input{'accountcode'};
    $r->add_attributes ( { Name => 'User-Name', Value => $input{'accountcode'} } );
    $r->add_attributes ( { Name => 'Password', Value => $params{'Password'} } ) if defined $params{'Password'};
}

$r->add_attributes ( { Name => 'h323-remote-address', Value => "$params{H323_RADDR}" } ) if defined $params{'H323_RADDR'};
$r->add_attributes ( { Name => 'h323-ivr-out', Value => "PortaBilling_Routing:$params{Routing}" } ) if defined $params{'Routing'};
$r->send_packet (ACCESS_REQUEST) and $type = $r->recv_packet;


if( !defined $type) {
    
    $error = $r->get_error;
    ($setuptime2,$setupms2) = gettimeofday();

    ###################################################################################
    #
    # Master is unavailable. Use db2 to authentificate and route the call
    #
    syslog('info', "Ast-Aut: Master not available => Alternative Authentification for $h323_id");
    closelog;


    openlog('ast-auth-alt', 'cons,pid', 'daemon');
    syslog('info', "Ast-Aut-Alt : Call $h323_id");

    load_config();
    $dbh = DBI->connect("DBI:mysql:$DB_NAME1;host=$DB_HOST", $DB_USER, $DB_PASS, {PrintError=>0});

    my @debug_data;

    # Check in which context we are
    switch ($params{'CustomerType'}) {
        #####################################
        #
        # SIP Authentication
        #
        case("SipAuth") {
            ($return_code,my $cli_has_credits, @debug_data) = alternative_acc_auth($username);
            ($return_code,$dest,$contacts) = define_routing_params($input{'dnid'}) if ($return_code == H323_CONTINUE);
            # check  if customer has credits in case of non-free call
            $return_code = H323_CREDIT_LIMIT if ($return_code == H323_CONTINUE and not $cli_has_credits);
        }
        #####################################
        #
        # Node Authentication
        #
        case("PeerAuth") {
            $return_code = alternative_ip_auth($username);
            if ($return_code == H323_CONTINUE) {
                $dest = $input{'dnid'};
                my @host = ([$LOCAL_HOST,'']);
                $contacts = \@host;
            }
        }
        #####################################
        #
        # IP Authentication
        #
        case("IpAuth"){
            ($return_code,my $void,@debug_data) = alternative_acc_auth($username);
            ($return_code,$dest,$contacts) = define_routing_params($input{'dnid'}) if ($return_code == H323_CONTINUE);
       }
    }
    $dbh->disconnect();

    set_routing_data($input{'callerid'}, $dest, $contacts) if ($return_code == H323_CONTINUE or $return_code == H323_FREE_CALL);

    
    $AGI->set_variable('h323-return-code', $return_code);

    ($setuptime3,$setupms3) = gettimeofday();


    my $debug_str;
    if (defined @debug_data) {
      $debug_str = "";
      foreach $d (@debug_data){
        $debug_str.="," unless $debug_str eq "";
        $debug_str.=$d;
      }
    } else {
      # create empty fields
      $debug_str = ",,,,,";
    }

    # Log Call ID, CLI, CLD, setup time, connection time, error reason, return code
    syslog('notice', ",$h323_id,$input{'callerid'},$username,$input{'dnid'},"
                   .time2str($date_format, $setuptime1).".$setupms1,"
                   .time2str($date_format, $setuptime2).".$setupms2,"
                   .time2str($date_format, $setuptime3).".$setupms3,$error,$return_code,"
                   .$params{H323_RADDR}.",".$params{'CustomerType'}.","
                   . $debug_str);
	
	
    $AGI->hangup() if $return_code != H323_CONTINUE && $return_code != H323_FREE_CALL && $hangup;
    $AGI->set_autohangup($credit_time) if $credit_time != -1;
    syslog('info', "Ast-Aut-Alt: return_code = $return_code");
    closelog;
    exit;

}

##########################################################################
# Treatment reply auth. radius packet from portaone server
my $dial_info = '';
my ($ActiveFolloMe,$TypeFollowMe);
my $Pred_expire=0;
my $ActivePrivacy=0;
for $a ($r->get_attributes) {
    $AGI->verbose("---- attr: name=$a->{'Name'} value=$a->{'Value'}", 4);
    $AGI->set_variable($a->{'Name'}, $a->{'Value'});
    $return_code = $a->{'Value'} if $a->{'Name'} eq 'h323-return-code';
    $credit_time = List::Util::min($max_duration,$a->{'Value'}) if $a->{'Name'} eq 'h323-credit-time';
    if( $a->{'Name'} eq 'Cisco-AVPair' && $a->{'Value'} =~ /^h323-ivr-in=PortaBilling_Routing:/ ) {
        $AGI->verbose("  -- AGI full route: $a->{'Value'}", 4);
        #TODO this regexp is very important, double-check
        $a->{'Value'} =~ /^h323-ivr-in=PortaBilling_Routing:\s*([^@]*)@([^;]*);?(.*)$/;
        my ($dest, $contact) = ($1, $2);
        if ($a->{'Value'} =~ /^h323-ivr-in=PortaBilling_Routing:.*;cli=;/){
           $ActivePrivacy=1;
           }
        my %route_info;
        my(@pairs) = split(/[&;]/,$3);
        my($param,$value);
        foreach (@pairs) {
            ($param,$value) = split('=',$_,2);
            $param = unescape($param);
            $value = unescape($value);
            $route_info{$param}=$value;
            }
        my $cipher = Crypt::CBC->new({ 'key'            => $AGI->get_variable('RADIUS_Secret'),
                                       'cipher'         => 'DES',
                                       'iv'             => '$KJh#(}q',
                                       'regenerate_key' => 1, # default  true
                                       'padding'        => 'oneandzeroes',
                                       'prepend_iv'     => 0
                                    });
        my ($authuser, $secret) = split(/\0/, $cipher->decrypt(pack("H*",$route_info{'auth'})) );


        $ActiveFolloMe=1 if ($route_info{'access-code'} eq 'FOLLOWME');
        switch ($route_info{'g-hunt'}) {
            case 'all'  { $TypeFollowMe='Simultaneous';}
            case 'seq'  { $TypeFollowMe='Sequential' ;}
            }
        if ($route_info{'expires'}) {
            $skip_route++;
            $Pred_expire=$route_info{'expires'};
            }
        if($dest ne '' and $contact ne '' and ($Pred_expire or $ActivePrivacy)) {
            if (!$Pred_expire and $ActivePrivacy){
               $Pred_expire=30;
               $skip_route++;
               }
            @routes[$numroutes]=$params{'Routing'}."/".$dest."@".$contact;
            if (defined $route_info{'cli'}) { @clis[$numroutes]=$route_info{'cli'};}
            elsif($ActivePrivacy) {@clis[$numroutes]=$username;}
            @Join_Expire[$numroutes]=$skip_route;
            @Expire[$numroutes]=$Pred_expire;
            $numroutes++;
            $dial_info .= '&' if $dial_info ne '';
            $dial_info .= "$params{Routing}/$dest";
            $dial_info .= ":$secret" if defined $route_info{'auth'} && defined $secret;
            $dial_info .= ":$authuser" if defined $route_info{'auth'} && defined $authuser;
            $dial_info .= '@'.$contact;

            }
        else {
            $AGI->verbose("  -- AGI route skipped",4);
            }
        }
    if( $a->{'Name'} eq 'Cisco-AVPair' && $a->{'Value'} =~ /^h323-ivr-in=PortaBilling_CompleteNumber:/ ) {
        $routing_info = $a->{'Value'};
        $routing_info =~ s/^h323-ivr-in=PortaBilling_CompleteNumber://;
        $AGI->set_variable('DNID',$routing_info);
        }
    if( $a->{'Name'} eq 'Cisco-AVPair' && $a->{'Value'} =~ /^h323-ivr-in=PortaBilling_CLI:/ ) {
        $routing_info = $a->{'Value'};
        $routing_info =~ s/^h323-ivr-in=PortaBilling_CLI://;
        $auth_cli=$routing_info;
        }
    }

##########################################################################
# Routing call
$TypeFollowMe='List' if ($ActiveFolloMe && !$TypeFollowMe);
syslog('info', "Ast-Aut: ActiveFolloMe=$ActiveFolloMe -> $TypeFollowMe");
my @XSUM;

$AGI->set_variable('NumRoutes', $numroutes);
$AGI->set_variable('ActivePrivacy', $ActivePrivacy);
for ($i=0;$i<$numroutes;$i++) {
    my $cli=$auth_cli;
    $cli=@clis[$i] if defined @clis[$i];
    $AGI->verbose("  -- AGI set route $i: @routes[$i] CLI: $cli",4);
    if (@Join_Expire[$i]) {
        @XSUM[@Join_Expire[$i]]=0 if (!defined @XSUM[@Join_Expire[$i]]);
		if (@routes[$i] && @Expire[$i] && $cli) {
			$AGI->set_variable("XROUTE_@Join_Expire[$i]_@XSUM[@Join_Expire[$i]]", @routes[$i]);
			$AGI->set_variable("XEXP_@Join_Expire[$i]_@XSUM[@Join_Expire[$i]]", @Expire[$i]);
			$AGI->set_variable("XCLI_@Join_Expire[$i]_@XSUM[@Join_Expire[$i]]", $cli);
			syslog('info', "Ast-Aut: XROUTE_@Join_Expire[$i]_@XSUM[@Join_Expire[$i]]=@routes[$i] | XEXP_@Join_Expire[$i]_@XSUM[@Join_Expire[$i]]=@Expire[$i] | XCLI_@Join_Expire[$i]_@XSUM[@Join_Expire[$i]]=$cli") ;
			@XSUM[@Join_Expire[$i]]++;
			}
        }    
    }

my ($k,$II) = (0,0);
foreach my $vl (@XSUM) {
    if ($vl) {
        syslog('info', "Ast-Aut: XSUM_$k = $vl");
        $AGI->set_variable("XSUM_$k", $vl);
		if (!$II) {
			$II=$k;
			$AGI->set_variable("II", $II);
			syslog('info', "Ast-Aut: II = $II");
			}
        }
    $k++;
    }
$k = $k - 1 ;
$AGI->set_variable("XTOT", $k);
syslog('info', "Ast-Aut: XTOT = $k");
$AGI->set_variable('ActiveFolloMe', $ActiveFolloMe);
$AGI->set_variable('RADIUS_Status', 'OK');
$AGI->set_variable('Dial_Info', $dial_info) if $dial_info ne '';

syslog('info', "AGI dial info : " . $dial_info);


$AGI->verbose("  -- AGI full dial info: $dial_info",4);
$AGI->hangup() if $return_code != 0 && $return_code != 13 && $hangup;
$AGI->set_autohangup($credit_time) if $credit_time != -1;
syslog('info', "Ast-Aut: return_code = $return_code");
closelog;
exit;
##########################################################################
#                                  END                                   #
##########################################################################
